//manipulace se stopama
function GetADrakeDbPath:string;
begin
  Result:=GetPhoneDir()+'\_adrake.db3';
end;

function GpxHeader(sName:string):string;
begin
  Result:='<?xml version="1.0" encoding="UTF-8" standalone="no" ?>'+CRLF
         +'<gpx xmlns="http://www.topografix.com/GPX/1/1" creator="aDrakeSync" version="1.1">'+CRLF
         +'  <trk>'+CRLF
         +'    <name><![CDATA['+AnsiToUtf(sName) + ']]></name>'+CRLF
         +'    <trkseg>'+CRLF;
end;

function GpxFooter():string;
begin
  Result:='    </trkseg>'+CRLF
         +'  </trk>'+CRLF
         +'</gpx>';
end;

function KmlHeader1: string;
begin
  Result := '<?xml version="1.0" encoding="UTF-8"?>'+CRLF+
            '<kml xmlns="http://earth.google.com/kml/2.2">'+CRLF+
            '  <Document>'+CRLF+
            '    <name><![CDATA[aDrake]]></name>'+CRLF+
            '    <open>1</open><visibility>1</visibility>'+CRLF;
end;

function KmlHeader2(Name, Color:string):string;
begin
  Result := '    <Placemark>'+CRLF+
            '      <Snippet></Snippet>'+CRLF+
            '      <Style><LineStyle><color>'+Color+'</color><width>4</width></LineStyle></Style>'+CRLF+
            '      <name><![CDATA['+AnsiToUtf(Name)+']]></name>'+CRLF+
            '      <description><![CDATA[a:Drake track]]></description>'+CRLF+
            '      <MultiGeometry>'+CRLF+
            '        <LineString>'+CRLF+
            '          <tessellate>1</tessellate>'+CRLF+
            '          <altitudeMode>clampToGround</altitudeMode>'+CRLF+
            '          <coordinates>'+CRLF;
end;

function KmlFooter2:string;
begin
  Result := '          </coordinates>'+CRLF+
            '        </LineString>'+CRLF+
            '      </MultiGeometry>'+CRLF+
            '    </Placemark>'+CRLF;
end;

function KmlFooter1():string;
begin Result:='  </Document>'+CRLF+'</kml>'; end;

function PltHeader(sRgbColor:string):string;
begin
  if(sRgbColor='') then sRgbColor:='255'; //cervena barva, v dec formatu
  Result:='OziExplorer Track Point File Version 2.0'+CRLF
         +'WGS 84'+CRLF
         +'Altitude is in Feet'+CRLF
         +'Reserved 3'+CRLF
         +'0,2,'+sRgbColor+',a:Drake Track Log,1'{tady jete pole, ktere ale aDrake nedava:
                                                 track type 0(=normal)/10(=polygon)/20(=alarm zone)
                                                 track fill style 0(=solid)/1(=clear)/2(=Bdiagonal)
                                                   /3(=Fdiagonal)/4(=Cross)/5(=DiagCross)
                                                   /6(=Horizontal)/7(=Vertical)
                                                 track fill color (RGB, v dec formatu) }
           +CRLF
         +'0'+CRLF;
end;

function PltFooter:string;
begin Result:=''; end;

function MakeTrackDatetime(sDt,sTm:string): TDateTime;
var s:string;
begin
  s:=Copy('000000',1,6-Length(sTm))+sTm;    // doplnime cas na format hhmmss
  Result:=IntToDate(sDt)+EncodeTime(StrToInt(copy(s,1,2)),StrToInt(copy(s,3,2)),StrToInt(copy(s,5,2)),0);
  //{$ifdef DEBUG_HELPER} LDH('..sDt='+sDt+', sTm='+sTm+' >>> '+FloatToStr(Result)); {$endif}
end;

function TextToDt(sDateTime:string):TDateTime;
var s1,s2,s3:string;
begin
  //prevadi format dd.mm.rrrr hh:mm:ss na TDateTime
  s1:=Fetch(sDateTime,'.');
  s2:=Fetch(sDateTime,'.');
  s3:=Fetch(sDateTime,' ');
  Result:=EncodeDate(StrToInt(s3), StrToInt(s2), StrToInt(s1));
  s1:=Fetch(sDateTime,':');
  s2:=Fetch(sDateTime,':');
  s3:=sDateTime;
  Result:=Result+EncodeTime(StrToInt(s1), StrToInt(s2), StrToInt(s3),0);
end;

function PltAlt(tab:TSQLiteTable; idx:integer):string;
var eAlt:double;
begin
  Result:='-777'; //pokud neni hodnota zadana, je -777
  if(tab.FieldIsNull(idx)) then exit;
  eAlt:=tab.FieldAsDouble(idx)*3.2808399; //prevedeme metry na feety
  //{$ifdef DEBUG_HELPER} LDH('..alt='+FloatToStr(eAlt)); {$endif}
  Result:=ReplaceString(FormatFloat('#.#',eAlt),',','.');
end;

function PltDate(tab:TSQLiteTable; idxDate,idxTime:integer):string;
var sDate,sTime:string;
    //dt:TDateTime;
begin
  Result:=''; //pokud neni hodnota prazdna
  if((tab.FieldIsNull(idxDate)) OR (tab.FieldIsNull(idxTime))) then exit;
  sDate:=tab.FieldAsString(idxDate);
  sTime:=tab.FieldAsString(idxTime);
  Result:=FloatToStr(MakeTrackdateTime(sDate,sTime));
end;

function GetTrackId:string;
var s:string;
begin
  s:=frmMain_sTrackName.Caption;
  Result:=Fetch(s,':');
end;

function GetCoordsFromDb(sId,sType:string):string; //nacte z databaze souradnice bodu stopy
var sSql,sFile,sAlt,sDate:string;
    DB:TSqliteDatabase;
    tab:TSQLiteTable;
begin
  {$ifdef DEBUG_HELPER} LDH('..Cteni souradnic stopy sId='+sId+' ve formatu '+sType); {$endif}
  frmMain_memoLog.Lines.Add(LOG_TAB+'GetCoordsFromDb: '+sId);
  sFile:=GetADrakeDbPath();
  if(not FileExists(sFile)) then begin
    ShowMessage(_('ERR: neexistuje databazovy soubor')+' '+sFile);        //##_EN_##
    exit;
  end;
  DB:=TSqliteDatabase.Create(sFile);
  try
    sSql:='SELECT lon, lat, alt, date, time FROM track_log WHERE trackid="'+sId+'" ORDER BY key;';
    tab:=DB.GetTable(sSql,false);
    try
      while not tab.eof do begin
        case sType of
        'GPX': begin
                 sDate:=tab.FieldAsString(4);
                 sDate:=Copy('000000',1,6-Length(sDate))+sDate;
                 insert(':',sDate,3);
                 insert(':',sDate,6);
                 sDate:=tab.FieldAsString(3)+'T'+sDate+'Z';
                 insert('-',sDate,5);
                 insert('-',sDate,8);
                 Result:=Result+'     <trkpt lat="'+tab.FieldAsString(1)+'" lon="'+tab.FieldAsString(0)+'">'
                       +'<ele>'+tab.FieldAsString(2)+'</ele><time>'+sDate+'</time></trkpt>'+CRLF;
               end;
        'PLT': //lat,lon,0 (=normal, 1=break),alt (vzdy ve feetech, -777=neplatna hodnota,datum,{datum jako string},{cas jako string}
               // datum a cas ve string formatu aDrake nedava (OziExplorer je ignoruje)
               //datum a cas je v Delphi formatu (desetinne cislo od 30.12.1899
               begin
               sAlt:=PltAlt(tab,2);
               sDate:=PltDate(tab,3,4);
               Result:=Result+tab.FieldAsString(1)+','+tab.FieldAsString(0)+',0,'
                 +sAlt+','+sDate+',,'+CRLF;
               ;
               end;
        'KML': Result:=Result+tab.FieldAsString(0)+','+tab.FieldAsString(1)+','+tab.FieldAsString(2)+CRLF;
        end;
        tab.Next();
      end;
    finally
      if(tab<>nil) then tab.Free();
    end;
  finally
    if(DB<>nil) then DB.Free();
  end;
  {$ifdef DEBUG_HELPER} LDH('..Nacteno'); {$endif}
end;

procedure lTracks_DrawItem(list: TWinControl; Index: Integer; Rect: TRect;
  State: byte {podle dokumentace je typu TOwnerDrawState, ale v enginu je byte});
var
  sText,sId,sName,sDate,sTime,sPoints,sLength:string;
begin
  sText:=(list as TListBox).Items[Index];
  (list as TListBox).Canvas.Brush.Color:=BKGCOLOR_FOCUSED;
  if((State AND 1)=1) then begin
    (list as TListBox).Canvas.Brush.Color:=BKGCOLOR_SELECTED;
    (list as TListBox).Canvas.Font.Color:=COLOR_SELECTED;
  end;
  (list as TListBox).Canvas.FillRect(Rect);
  sId:=Fetch(sText,',"');
  sName:=Fetch(sText,'",');
  sDate:=Fetch(sText,',');
  sTime:=Fetch(sText,','); if(Length(sTime)<4) then sTime:='0'+sTime;
  sPoints:=Fetch(sText,',');
  sLength:=Fetch(sText,',');
  (list as TListBox).Canvas.TextOut(Rect.Left+3,Rect.Top,sId);
  (list as TListBox).Canvas.TextOut(Rect.Left+30,Rect.Top,Copy(sDate,7,2)+'.'+Copy(sDate,5,2)+'.'+Copy(sDate,1,4));
  (list as TListBox).Canvas.TextOut(Rect.Left+100,Rect.Top,Copy(sTime,1,2)+':'+Copy(sTime,3,2));
  (list as TListBox).Canvas.TextOut(Rect.Left+150,Rect.Top,sLength+' m');
  (list as TListBox).Canvas.TextOut(Rect.Left+230,Rect.Top,sName);
end;

function SetDecimal(str:string; num:integer):string; //odrizne prebytecna desetinna mista
var i:integer;
begin
  Result:=str;
  i:=Pos('.',str)
  if(i>=1) then Result:=Copy(str,1,i+num);
end;

procedure lPoints_DrawItem(list: TWinControl; Index: Integer; Rect: TRect;
  State: byte {podle dokumentace je typu TOwnerDrawState, ale v enginu je byte});
var
  sText,sKey,sLat,sLon,sTime,sAlt,sAccuracy,sSpeed,sBearing:string;
begin
  sText:=(list as TListBox).Items[Index];
  (list as TListBox).Canvas.Brush.Color:=BKGCOLOR_FOCUSED;
  if((State AND 1)=1) then begin
    (list as TListBox).Canvas.Brush.Color:=BKGCOLOR_SELECTED;
    (list as TListBox).Canvas.Font.Color:=COLOR_SELECTED;
  end;
  (list as TListBox).Canvas.FillRect(Rect);
  //sIdx:=Fetch(sText,',');
  sKey:=Fetch(sText,',');
  sLat:=Fetch(sText,',');
  sLon:=Fetch(sText,',');
  sTime:=Fetch(sText,',');
  sAlt:=SetDecimal(Fetch(sText,','),1);
  sAccuracy:=SetDecimal(Fetch(sText,','),1);
  sSpeed:=Fetch(sText,',');
  sBearing:=Fetch(sText,',');
  (list as TListBox).Canvas.TextOut(Rect.Left+3,Rect.Top,sKey);
  (list as TListBox).Canvas.TextOut(Rect.Left+55,Rect.Top,sLat);
  (list as TListBox).Canvas.TextOut(Rect.Left+135,Rect.Top,sLon);
  (list as TListBox).Canvas.TextOut(Rect.Left+215,Rect.Top,sTime);
  (list as TListBox).Canvas.TextOut(Rect.Left+345,Rect.Top,sAlt);
  (list as TListBox).Canvas.TextOut(Rect.Left+390,Rect.Top,sAccuracy);
  (list as TListBox).Canvas.TextOut(Rect.Left+455,Rect.Top,sSpeed);
  (list as TListBox).Canvas.TextOut(Rect.Left+510,Rect.Top,sBearing);
end;

procedure EnableTracksButtons;
var b,bSel,bMtp,bTrackDb:boolean;
    i,iSel:integer;
begin
  b:=(IsIp(false));
  bMtp:=(IsMtp(false));
  bTrackDb:=({(frmMain_bTrackViewFromDb.Enabled) or} (frmMain_bTrackViewFromDb.Checked));
  iSel:=0;
  for i:=0 to frmMain_lTracks.Items.Count-1 do begin
    if(frmMain_lTracks.Selected[i]) then Inc(iSel);
  end;
  bSel:=(iSel>0);
  //{$ifdef DEBUG_HELPER} LDH('bSel='+IntToStr(bSel as integer)); {$endif}
  frmMain_butTrackShow.Enabled:=bSel;
  frmMain_butTrackSave.Enabled:=bSel;
  frmMain_sTrackFile.Visible:=false;
  frmMain_sDbFile.Visible:=false;
  frmMain_sTrackNameLab.Visible:=bSel;   frmMain_sTrackName.Visible:=bSel;
  frmMain_sTrackCountLab.Visible:=bSel;  frmMain_sTrackCount.Visible:=bSel;
  frmMain_sTrackDistLab.Visible:=bSel;   frmMain_sTrackDist.Visible:=bSel;
  frmMain_sTrackStartStop.Visible:=((bSel) and (bTrackDb));
  frmMain_sTrackElePlusMinusLab.Visible:=((bSel) and (bTrackDb));
    frmMain_sTrackElePlusMinus.Visible:=((bSel) and (bTrackDb));
  frmMain_sTrackSpeedLab.Visible:=((bSel) and (bTrackDb));
    frmMain_sTrackSpeed.Visible:=((bSel) and (bTrackDb));
  frmMain_sTrackEleLab.Visible:=((bSel) and (bTrackDb));
    frmMain_sTrackEle.Visible:=((bSel) and (bTrackDb));
  frmMain_butTrackPoints.Enabled:=((bSel) and (bTrackDb));
  frmMain_butTrackRename.Enabled:=((bSel) and (bTrackDb))
  frmMain_butTrackDel.Enabled:=((bSel) and (bTrackDb))
  frmMain_butTracksJoin.Enabled:=((iSel>1) and (bTrackDb));
  frmMain_butTrackSplit.Enabled:=((bSel) and (bTrackDb));
  frmMain_butTrackToAD.Enabled:=((bMtp) and (bTrackDb));
end;

procedure ShowTrackSummary;
var sText:string;
    sFile,sId,sSql:string;
    DB:TSqliteDatabase;
    tab:TSQLiteTable;
    i,iEleStart,iEleStop,iEleLast,iEleMin,iEleMax,iElePlus,iEleMinus:integer;
    sDistance,sDtStart,sDtStop,sTmStart,sTmStop:string;
    b:boolean;
    dtStart,dtStop:TDateTime;
    eSpeed:extended;
begin
  sText:=frmMain_lTracks.Items[frmMain_lTracks.ItemIndex];
  sId:=Fetch(sText,',"');
  frmMain_sTrackName.Caption:=sId+': '+Fetch(sText,'",');
  {sDate:=}Fetch(sText,',');
  {sTime:=}Fetch(sText,','); //if(Length(sTime)<4) then sTime:='0'+sTime;
  frmMain_sTrackCount.Caption:=Fetch(sText,',');
  sDistance:=ReplaceString(Fetch(sText,','),'.0','')
  frmMain_sTrackDist.Caption:=sDistance+' m';
  //pokud je ziskany databazovy soubor, zkusime nacist ostatni hodnoty
  sFile:=GetADrakeDbPath();
  if(not FileExists(sFile)) then exit;
  iEleMin:=99999; iEleMax:=-99999; iEleStart:=-99999; iEleStop:=0;
  sDtStart:='';
  if(not FileExists(sFile)) then begin
    {$ifdef DEBUG_HELPER} LDH('ERR: neexistuje databazovy soubor '+sFile); {$endif}
    {$ifdef DEBUG_HELPER} LDH('ERR: asi neni spravne identifikovano jmeno telefonu'); {$endif}
  end
  else begin //pokud soubor existuje
    DB:=TSqliteDatabase.Create(sFile);
    try
      sSql:='SELECT date, time, lat, lon, alt, accuracy, speed, bearing FROM track_log WHERE trackid="'
        +sId+'" ORDER BY trackid,key;';
      tab:=DB.GetTable(sSql,false);
      try
        while not tab.eof do begin
          if(sDtStart='') then begin
            b:=tab.FieldIsNull(4);
            {$ifdef DEBUG_HELPER} LDH('nezadana nadmoska vyska='+IntToStr(b as integer)); {$endif}
            iEleLast:=iEleStart;
            sDtStart:=tab.FieldAsString(0);
            sTmStart:=tab.FieldAsString(1);
          end;
          //minimalni, maximalni, nastoupana, naklesana vyska
          if(not tab.FieldIsNull(4)) then begin
            iEleStop:=tab.FieldAsInteger(4);
            if(iEleStart=-99999) then begin
              {$ifdef DEBUG_HELPER} LDH('pocatecni nadmoska vyska='+IntToStr(iEleStop)); {$endif}
              iEleStart:=iEleStop;
              iEleLast:=iEleStart;
            end;
            i:=iEleStop-iEleLast;
            if(i>0) then iElePlus:=iElePlus+i;
            if(i<0) then iEleMinus:=iEleMinus-i;
            if(iEleStop>iEleMax) then iEleMax:=iEleStop;
            if(iEleStop<iEleMin) then iEleMin:=iEleStop;
            iEleLast:=iEleStop;
          end;
          //cas konce
          sDtStop:=tab.FieldAsString(0);
          sTmStop:=tab.FieldAsString(1);
          //rychlosti
          tab.Next();
        end;
        dtStart:=MakeTrackDatetime(sDtStart,sTmStart);
        dtStop:=MakeTrackDatetime(sDtStop,sTmStop);
        dtStop:=dtStop-dtStart;
        if(dtStop=0) then eSpeed:=0
        else eSpeed:=StrToFloat(sDistance)/(dtStop*24)/1000.;
        frmMain_sTrackSpeed.Caption:=ReplaceString(FormatFloat('0.0',eSpeed),',','.')+' Km/h';
        sDtStart:=Copy(sDtstart,7,2)+'.'+Copy(sDtstart,5,2)+'.'+Copy(sDtstart,1,4)
          +' '+Copy(sTmStart,1,2)+':'+Copy(sTmStart,3,2)+':'+Copy(sTmStart,5,2);
        sDtStop:=Copy(sDtstart,7,2)+'.'+Copy(sDtStop,5,2)+'.'+Copy(sDtStop,1,4)
          +' '+Copy(sTmStop,1,2)+':'+Copy(sTmStop,3,2)+':'+Copy(sTmStop,5,2);
        frmMain_sTrackStartStop.Caption:=sDtStart+'  -  '+sDtStop;
        frmMain_sTrackEle.Caption:=IntToStr(iEleStart)+'/'+IntToStr(iEleStop)+' - '
          +IntToStr(iEleMin)+'/'+IntToStr(iEleMax);
        frmMain_sTrackElePlusMinus.Caption:='+'+IntToStr(iElePlus)+'/-'+IntToStr(iEleMinus);
      finally
        if(tab<>nil) then tab.Free();
      end;
    finally
      if(DB<>nil) then DB.Free();
    end;
  end;
end;

procedure lTracksOnClick(Sender: TObject);
begin
  ShowTrackSummary();
  EnableTracksButtons();
end;

function FillTracksFromDb(sFile:string):string;
  //naplni string seznamem stop z databaze jakoby jej dostal od a:Drake
var sSql,s:string;
    DB:TSqliteDatabase;
    tab:TSQLiteTable;
begin
  Result:='';
  //{$ifdef DEBUG_HELPER} LDH('Aktualizace seznamu stop z '+sFile); {$endif}
  if(not FileExists(sFile)) then exit;
  frmMain_sDbFile.Caption:=sFile;
  DB:=TSqliteDatabase.Create(sFile);
  try
    sSql:='SELECT trackid, name, date, time, size, odo FROM track_h'
      +' ORDER BY trackid desc;';
    tab:=DB.GetTable(sSql,false);
    try
      while not tab.eof do begin
        //vzdalenost je s hodne velkou presnosti, tak nechame cele metry
        s:=FormatFloat('#',tab.FieldAsDouble(5));
        if(s='') then s:='0';
        Result:=Result+tab.FieldAsString(0)+',"'+tab.FieldAsString(1)+'",'
          +tab.FieldAsString(2)+','+tab.FieldAsString(3)+','
          +tab.FieldAsString(4)+','+s+CRLF;
        tab.Next();
      end;
    finally
      if(tab<>nil) then tab.Free();
    end;
  finally
    if(DB<>nil) then DB.Free();
  end;
end;

procedure butTracksGetOnClick(Sender: TObject);
var s,sDev,response:string;
    sSrc,sTrg:unicodestring;
    i:integer;
begin
  {$ifdef DEBUG_HELPER} LDH('butTracksGetOnClick(): start'); {$endif}
  frmMain_memoLog.Lines.Add('GetTracks: start');
  frmMain_butTracksGet.Cursor:=crHourglass;
  frmMain_bTrackViewFromDb.Enabled:=false;
  frmMain_bTrackViewFromDb.Checked:=false;
  frmMain_lTracks.Items.Text:='';
  if(IsIP(false)) then begin
    HttpAsk('GET',GetFceUrl('tracks'),'','',response);
    if(not IsConnected(response)) then begin
      frmMain_memoLog.Lines.Add('GetTracks: ERR');
      frmMain_butTracksGet.Cursor:=crDefault;
      exit;
    end;
  end;
  //pokud je k dispozici MTP, stahneme soubor _adrake.db3
  if(IsMtp(false)) then begin
    frmMain_memoLog.Lines.Add(LOG_TAB+'get _adrake.db3');
    s:=frmMain_sMtpPath.Caption;
    sDev:=Fetch(s,':');
    ML_SetDeviceName(sDev);
    sSrc:=s+'\_adrake.db3';
    sTrg:=GetADrakeDbPath();
    DeleteFile(sTrg);
    //{$ifdef DEBUG_HELPER} LDH('  sSrc='+sSrc+', sTrg='+sTrg); {$endif}
    i:=ML_GetFile(sSrc,sTrg,false);
    {$ifdef DEBUG_HELPER} LDH('  stav cteni "_adrake.db3": '+IntToStr(i)+' '+ML_GetErrorMessage()); {$endif}
    if(not IsIP(false)) then begin
      response:=FillTracksFromDb(sTrg);
    end;
    frmMain_bTrackViewFromDb.Enabled:=true;
  end;
  GeoBusyKind(_('Done'));
  frmMain_memoLog.Lines.Add('GetTracks: ok');
  StringToFile(response,GetPhoneDir()+'\tracks.txt');
  frmMain_lTracks.Items.Text:=response;
  frmMain_butTracksGet.Cursor:=crDefault;
  EnableTracksButtons();
end;

procedure butTrackSaveOnClick(Sender: TObject);
var response,sText,sId,sDate,sTime,sName,sFormat:string;
    i,iSel:integer;
begin
  {$ifdef DEBUG_HELPER} LDH('butTracksSaveOnClick(): start'); {$endif}
  frmMain_memoLog.Lines.Add('SaveTrack: start');
  frmMain_sTrackFile.Caption:='';
  frmMain_butTrackSave.Cursor:=crHourglass;
  iSel:=0;
  for i:=0 to frmMain_lTracks.Items.Count-1 do begin
    //zpracujeme vsechny vybrane stopy
    if(not frmMain_lTracks.Selected[i]) then continue;
    sText:=frmMain_lTracks.Items[i];
    Inc(iSel);
    sId:=Fetch(sText,',"');
    sName:=Fetch(sText,'",');
    frmMain_memoLog.Lines.Add(LOG_TAB+'SaveTrack: '+sId+' - '+sName+'...');
    sDate:=Fetch(sText,',');
    sTime:=Fetch(sText,','); if(Length(sTime)<4) then sTime:='0'+sTime;
    sText:=GetFceUrl('tracks('+sId+')/'+sFormat);
    case frmMain_rTracksFormat.ItemIndex  of
      0: sFormat:='gpx';
      1: sFormat:='kml';
      2: sFormat:='plt';
    end;
    if(frmMain_bTrackViewFromDb.Checked) then begin
      case sFormat of
        'gpx':response:=GpxHeader(sName)+GetCoordsFromDb(sId,UpperCase(sFormat))+GpxFooter();
        'kml': response:=KmlHeader1()+KmlHeader2(sName,'ffffff00'{svetle modra})
                 +GetCoordsFromDb(sId,UpperCase(sFormat))+KmlFooter2()+KmlFooter1();
        'plt': begin
                {$ifdef DEBUG_HELPER} LDH('..Nacteni PLT stopy z databaze sId='+sId); {$endif}
                response:=PltHeader('255');
                response:=response+GetCoordsFromDb(sId,UpperCase(sFormat))+PltFooter();
                //{$ifdef DEBUG_HELPER} LDH('..response '+response); {$endif}
               end;
      end;
    end
    else begin
      {$ifdef DEBUG_HELPER} LDH('sText='+sText); {$endif}
      HttpAsk('GET',sText,'','',response);
      if(not IsConnected(response)) then begin
        frmMain_memoLog.Lines.Add('SaveTrack: ERR');
        frmMain_butTrackSave.Cursor:=crDefault;
        exit;
      end;
    end;
    frmMain_sTrackFile.Caption:=GetPhoneDir()+'\Track_'+sId+'_'+sName+'.'+sFormat;
    StringToFile(response,frmMain_sTrackFile.Caption);
  end;
  {$ifdef DEBUG_HELPER} LDH('Stopy ulozeny do souboru'); {$endif}
  GeoBusyKind(_('Done'));
  frmMain_memoLog.Lines.Add('SaveTrack: ok');
  frmMain_butTrackSave.Cursor:=crDefault;
end;

function GetPathFromRegEdit(sHKEY, sKey:string):string;
var aReg:TRegistry;
begin
  //automaticky zkusi i pripadne WOW6432Node na 32bitovem pocitaci vymazat
  //a pripadne doplni i "%1" jako priznak parametru
  {$ifdef DEBUG_HELPER} LDH('Ziskani prikazoveho radku z '+sHKEY); {$endif}
  Result:='';
  try
    aReg:=TRegistry.Create();
    aReg.Access:=131097; //nastavime KEY_READ
    aReg.LazyWrite:=false;
    case Fetch(sHKEY,'\') of
    'HKCR','HKEY_CLASSES_ROOT':     aReg.RootKey:=$80000000;
    'HKCU','HKEY_CURRENT_USER':     aReg.RootKey:=$80000001;
    'HKLM','HKEY_LOCAL_MACHINE':    aReg.RootKey:=$80000002;
    'HKU', 'HKEY_USERS'         :   aReg.RootKey:=$80000003;
    'HKPD','HKEY_PERFORMANCE_DATA': aReg.RootKey:=$80000004;
    'HKCC','HKEY_CURRENT_CONFIG'  : aReg.RootKey:=$80000005;
    'HKDD','HKEY_DYN_DATA'        : aReg.RootKey:=$80000006;
    else aReg.RootKey:=0;
    end;
    aReg.OpenKey(sHKEY,true);
    Result:=aReg.ReadString(sKey);
    if(Result='') then begin  //pokud jsme na 32bit systemu, zkusime jeste zakladni
      aReg.CloseKey();
      aReg.OpenKey(ReplaceString(sHKEY,'WOW6432Node',''),true);
      Result:=aReg.ReadString(sKey);
    end;
    if((Pos('%1',Result)<1) and (Result<>'')) then Result:=Result+' "%1"';
    aReg.CloseKey();
  finally
    aReg.Free();
  end;
  {$ifdef DEBUG_HELPER} LDH('1. Result='+Result); {$endif}
end;

procedure FillTracksShow; //naplni Combobox s programy a weby pro zobrazeni podle INI souboru
var ini:TMemIniFile;
    s0,s,sName:string;
    i:integer;
begin
  s0:=frmMain_cbTrackShow.Items[0];
  frmMain_cbTrackShow.Items.Text:='';
  ini:=TMemIniFile.Create(GetMyIni());
  try
    ini.ReadSectionValues('ShowTracks',frmMain_cbTrackShow.Items);
    for i:=0 to frmMain_cbTrackShow.Items.Count-1 do begin
      s:=frmMain_cbTrackShow.Items[i];
      sName:=Fetch(s,'=');
      frmMain_cbTrackShow.Items[i]:=sName;
    end;
  finally
    ini.Free();
  end;
  frmMain_cbTrackShow.Items.Insert(0,s0);
  frmMain_cbTrackShow.ItemIndex:=0;
end;

procedure butTracksFromFileOnClick(Sender: TObject);
var s,sPath:string;
begin
  {$ifdef DEBUG_HELPER} LDH('butFnFromFileOnClick'); {$endif}
  if(not SaveFnIfNeed()) then exit; //pokud je treba, ulozime zmeny
  //pokud je pozadovano, rekneme si o soubor a nacteme jej
  if(not OpenDlg(_('Open and view file with trackss'), //title
               GetMyDir(),               //InitDir
              'ofFileMustExist,ofHideReadOnly', //options
              _('Seznam stop')+'|tracks.txt;_adrake.db3'    //filtr
              +'|'+_('All files')+'  (*.*)|*.*'
              ,
              sPath)) then exit;
  //obslouzime podle pripony
  frmMain_bTrackViewFromDb.Enabled:=false;
  frmMain_bTrackViewFromDb.Checked:=false;
  s:=Copy(sPath,Length(sPath)-3,4);
  if(s='.txt') then s:=FileToString(sPath)
  else begin
    s:=FillTracksFromDb(sPath);
    //zjistime jmeno "telefonu"
    sPath:=ReplaceString(sPath,GetMyDir()+'\','');
    if(FileExists(GetMyDir()+'\'+sPath)) then begin //poznacime jmeno telefonu
      frmMain_sServerName.Caption:=Fetch(sPath,'\');
    end;
    //frmMain_bTrackViewFromDb.Enabled:=true; //neni mozne zmenit, protoze neni pripojeni k AD
    frmMain_bTrackViewFromDb.Checked:=true;
  end;
  frmMain_memoLog.Lines.Add('GetTracksFromFile: ok');
  frmMain_lTracks.Items.Text:=s;
  EnableTracksButtons();
end;

procedure butTrackShowOnClick(Sender: TObject);
var s,sPath,sPrg,sKey,sHKEY,sFile,sId,sKml,sColor,sTrackName:string;
    ini:TMemIniFile;
    i,iSel:integer;
begin
  iSel:=0;
  for i:=0 to frmMain_lTracks.Items.Count-1 do begin
    if(not frmMain_lTracks.Selected[i]) then continue;
    Inc(iSel);
  end;
  if(iSel<1) then exit;
  //if(frmMain_sTrackFile.Caption='') then frmMain_sTrackFile.Caption:='e:\Geocaching\GeoGet\script\GgDrake\LGE LG-H440n\Track_20171210_1652.gpx';
  butTrackSaveOnClick(nil);   //zbytecne: tohle ulozi vsechny vybrane stopy!
                              //ne jen tu jednu, kterou budu dal zpracovavat
  sFile:='';
  //pro KML zpracujeme stopu(y) do jednoho souboru s odlisenim barvy stop
  if(frmMain_rTracksFormat.ItemIndex=1) then begin
    {$ifdef DEBUG_HELPER} LDH('Spojeni stop do jednoho KML souboru'); {$endif}
    //spojime vsechny stopy v KML formatu
    frmMain_memoLog.Lines.Add('TrackJoin: start');
    sKml:=KmlHeader1();
    iSel:=0;
    for i:=0 to frmMain_lTracks.Items.Count-1 do begin
      if(not frmMain_lTracks.Selected[i]) then continue;
      //nastavime barvu, aby kazda stopa mela jinou
      case (iSel mod 10) of
        0 : sColor:='ff00ffff';     //zluta
        1 : sColor:='ff00ff00';     //zelena
        2 : sColor:='ffffff00';     //svetle modra
        3 : sColor:='ffff0000';     //tmave modra
        4 : sColor:='ffff00ff';     //fialova
        5 : sColor:='ff0000ff';     //cervena
        6 : sColor:='ff0067ff';     //oranzova
        7 : sColor:='ffbababa';     //sediva
        8 : sColor:='ffffffff';     //bila
        9 : sColor:='ff000000';     //cerna
        else sColor:='fff60d6b';     //zeleno sediva
      end;
      Inc(iSel);
      s:=frmMain_lTracks.Items[i];
      sId:=Fetch(s,',"');
      {$ifdef DEBUG_HELPER} LDH('sId='+sId); {$endif}
      sTrackName:=Fetch(s,'",');
      {$ifdef DEBUG_HELPER} LDH('sTrackName='+sTrackName); {$endif}
      frmMain_memoLog.Lines.Add(LOG_TAB+'Add: '+sTrackName);
      sKml:=sKml+KmlHeader2(sTrackName, UpperCase(sColor));
      sFile:=GetPhoneDir()+'\Track_'+sId+'_'+sTrackName+'.kml';
      //nacteme soubor, nechame z nej jen souradnice
      s:=FileToString(sFile);
      {$ifdef DEBUG_HELPER} LDH('stopa nactena'); {$endif}
      Fetch(s,'<coordinates>');
      sKml:=sKml+Fetch(s,'</coordinates>')+KmlFooter2();
      {$ifdef DEBUG_HELPER} LDH('stopa pripojena'); {$endif}
    end;
    sKml:=sKml+KmlFooter1();
    sFile:=GetPhoneDir()+'\Track_Joined.kml';
    frmMain_sTrackFile.Caption:=sFile; //aby se pracovalo s vyslednym souborem
    StringToFile(sKml,sFile);
    frmMain_memoLog.Lines.Add('TrackJoin: ok');
  end;

  {$ifdef DEBUG_HELPER} LDH('11'); {$endif}
  if(frmMain_cbTrackShow.ItemIndex=0) then begin  //zobrazit stopu v implicitnim programu podle pripony
    if(frmMain_sTrackFile.Caption<>'') then begin
      RunShell(frmMain_sTrackFile.Caption);
    end;
    exit;
  end;
  //zobrazit v konkretnim programu nebo na webu
  sPrg:=frmMain_cbTrackShow.Items[frmMain_cbTrackShow.ItemIndex];
  ini:=TMemIniFile.Create(GetMyIni());
  try
    sKey:=ini.ReadString('ShowTracks',sPrg,'');
  finally
    ini.Free();
  end;
  if(sKey='') then begin
    ShowMessage(_('ERR: Error while reading INI file'));
    exit;
  end;
  sHkey:=Fetch(sKey,':');
  //automaticky zkusi i pripadne WOW6432Node na 32bitovem pocitaci vymazat
  //a pripadne doplni i "%1" jako priznak parametru
  sPath:=GetPathFromRegEdit(sHKEY,sKey);
  if(sPath='') then ShowMessage(_('ERR: command to launch application not found')
    +' '+sPrg+'. '+_('Probably not installed..'))
  else begin
    sPath:=ReplaceString(sPath,'%1',frmMain_sTrackFile.Caption);
    if(RunExecNoWait(sPath)=false) then ShowMessage(_('ERR: Error while starting:')+CRLF+sPath);
  end;
end;

procedure rSplitOnClick(Sender: TObject);
begin
  frmTrack_pManually.Visible:=(frmTrack_rSplit.ItemIndex=0);
  frmTrack_pDistance.Visible:=(frmTrack_rSplit.ItemIndex=1);
  frmTrack_pTime.Visible:=(frmTrack_rSplit.ItemIndex=2);
  //ShowMessage('ItemIndex='+IntToStr(frmTrack_rSplit.ItemIndex));
end;

procedure rPointSelectTypeOnClick(Sender: TObject);
begin
  frmTrack_pPointAccuracy.Visible:=(frmTrack_rPointSelectType.ItemIndex=1);
  frmTrack_pPointTime.Visible:=(frmTrack_rPointSelectType.ItemIndex=2);
  frmTrack_butPointSelect.Visible:=(frmTrack_rPointSelectType.ItemIndex>=0);
  //jeste poloha tlacitka k oznaceni bodu
  if(frmTrack_pPointTime.Visible OR frmTrack_pPointAccuracy.Visible) then
    frmTrack_butPointSelect.Left:=frmTrack_pPointTime.Left+frmTrack_pPointTime.Width+5
  else frmTrack_butPointSelect.Left:=frmTrack_pPointTime.Left;
end;

procedure InitEditTrack(idx:integer); //inicializace formulare pro upravu stopy v zavislosti na typu upravy
var i,x:integer;
    sFile,sSql,sId:string;
    DB:TSqliteDatabase;
    //tab:TSQLiteTable;
begin
  case idx of
  0: begin //rozdeleni stopy
       x:=frmTrack_rSplit.Left;
       i:=frmTrack_rSplit.Width;
       frmTrack.Width:=i+2*(x+10);
       frmTrack.Height:=frmTrack_butSplit.Top+frmTrack_butSplit.Height+40 {zahlavi};
       frmTrack_pManually.Left:=frmTrack_rSplit.Left;
       frmTrack_pManually.Top:=frmTrack_rSplit.Top+frmTrack_rSplit.Height+3;
       frmTrack_pDistance.Left:=frmTrack_pManually.Left;
       frmTrack_pDistance.Top:=frmTrack_pManually.Top;
       frmTrack_pTime.Left:=frmTrack_pManually.Left;
       frmTrack_pTime.Top:=frmTrack_pManually.Top;
       frmTrack_butSplit.Left:=x+i/2-frmTrack_butSplit.Width/2;
       //nastavime spravnou zalozku jako aktivni
       frmTrack_pgTrack.ActivePage:=frmTrack_tabSplit;
       rSplitOnClick(nil);
     end;
  1:  begin //prehled bodu stopy
        frmTrack_pPointAccuracy.Left:=frmTrack_pPointTime.Left;
        frmTrack_pPointAccuracy.Top:=frmTrack_pPointTime.Top;
        //nastavime spravnou zalozku jako aktivni
        frmTrack_pgTrack.ActivePage:=frmTrack_tabPoints;
        x:=frmTrack_lPoints.Left;
        i:=frmTrack_lPoints.Width;
        frmTrack.Width:=i+2*(x+10);
        frmTrack.Height:=frmTrack_sSelectedDist.Top+frmTrack_sSelectedDist.Height+40 {zahlavi};
        frmTrack_sTrackName.Caption:='ID='+frmMain_sTrackName.Caption;
        sId:=GetTrackId();
        frmTrack_lPoints.Items.Text:='';
        rPointSelectTypeOnClick(nil);
        //nacist data z databaze
        {$ifdef DEBUG_HELPER} LDH('..Cteni bodu stopy sId='+sId); {$endif}
        //frmMain_memoLog.Lines.Add(LOG_TAB+'GetCoordsFromDb '+sId);
        sFile:=GetADrakeDbPath();
        if(not FileExists(sFile)) then begin
          ShowMessage(_('ERR: neexistuje databazovy soubor')+' '+sFile);
          exit;
        end;
        DB:=TSqliteDatabase.Create(sFile);
        try
          sSql:='SELECT key ||","|| lat ||","|| lon ||","|| '
               +' substr(date,7,2) ||"."|| substr(date,5,2) ||"."|| substr(date,1,4) ||" "|| ' //datum
                 +' substr("000000"||time,-6,2) ||":"|| substr("000000"||time,-4,2) ||":"|| substr("000000"||time,-2,2) ||","|| ' //cas
               +' alt ||","|| accuracy ||","|| speed ||","|| bearing '
               +' FROM track_log WHERE trackid="'+sId+'" ORDER BY key;';
          DB.GetTableStrings(sSql,frmTrack_lPoints.Items);
        finally
          if(DB<>nil) then DB.Free();
        end;
     end;
  end;
  frmTrack_pgTrack.Top:=-25; //skryjeme tlacitka zalozek
end;

procedure butTrackSplitOnClick(Sender: TObject);
begin
  InitEditTrack(0);
  frmTrack.ShowModal();
end;

procedure butTrackPointsOnClick(Sender: TObject);
begin
  InitEditTrack(1);
  frmTrack.ShowModal();
end;

function DbEditWarn:string;
begin
  Result:='Stopa byla v databazi upravena. V a:Drake se zmena projevi az po preneseni databaze do telefonu.';
end;

procedure UpdateTracksHeader(DB:TSqliteDatabase;sIds:string); //upravi pocet bodu stopy a vzdalenost
var sId,sSql:string;                 //seznam upravovanych stop, ID stop oddelene carkou
    n:integer;
    lat,lon,dist:extended;
    tab:TSQLiteTable;
begin
  while sIds<>'' do begin   //zpracujeme vsechny stopy v seznamu
    sId:=Fetch(sIds,',');
    n:=0;
    //zjistime pocet bodu stopy a delku stopy
    //vypocteme delku upravene stopy
    sSql:='SELECT lat,lon FROM track_log WHERE trackid='+sId+' ORDER BY key;';
    {$ifdef DEBUG_HELPER} LDH('..Vypocet vzdalenosti '+sSql); {$endif}
    tab:=DB.GetTable(sSql,false);
    try
      lat:=500;
      while not tab.eof do begin
        if(lat<>500) then dist:=dist+PointDistance(lat, lon, tab.FieldAsDouble(0), tab.FieldAsDouble(1));
        lat:=tab.FieldAsDouble(0);
        lon:=tab.FieldAsDouble(1);
        tab.Next();
        Inc(n);
      end;
      sSql:=ReplaceString(FloatToStr(dist*1000.),',','.');  //vypocitana hodnota je v km
      sSql:='UPDATE track_h SET size='+IntToStr(n)+', odo='+sSql+' WHERE trackid='+sId+' ;';
      {$ifdef DEBUG_HELPER} LDH('..Aktualizace hlavicky stopy '+sSql); {$endif}
      DB.ExecSQL(sSql);
    finally
      if(tab<>nil) then tab.Free();
    end;
  end;
end;

procedure butPointsDelOnClick(Sender: TObject); //vymazani vybranych bodu stopy
var s,sId,sSql,sFile:string;
    i,j,n:integer;
    DB:TSqliteDatabase;
begin
  j:=frmMain_lTracks.ItemIndex;
  sId:=GetTrackId();
  sSql:='';
  n:=0;
  for i:=0 to frmTrack_lPoints.Items.Count-1 do begin
    if(not frmTrack_lPoints.Selected[i]) then continue;
    s:=frmTrack_lPoints.Items[i];
    sSql:=sSql+Fetch(s,',')+CRLF;
    frmTrack_lPoints.Items.Delete(i);
    Dec(i);
    Inc(n);
  end;
  sSql:=ReplaceString(Trim(sSql),CRLF,',');
  sSql:='DELETE FROM track_log WHERE key in ('+sSql+');';
  //vymazat body
  sFile:=GetADrakeDbPath();
  if(not FileExists(sFile)) then begin
    ShowMessage(_('ERR: neexistuje databazovy soubor')+' '+sFile);
    exit;
  end;
  DB:=TSqliteDatabase.Create(sFile);
  try
    DB.Start('GgDrake',''); //zahajime transakci
    {$ifdef DEBUG_HELPER} LDH('..Mazani bodu z databaze '+sSql); {$endif}
    DB.ExecSQL(sSql);
    UpdateTracksHeader(DB,sId); //upravit pocet bodu a vzdalenost
    DB.Commit('GgDrake');//ukoncime transakci
    UpdateTracksHeader(DB,sId);
  finally
    if(DB<>nil) then DB.Free();
  end;
  s:=_('Pocet vymazanych bodu:')+' '+IntToStr(n);
  frmMain_memoLog.Lines.Add(s);
  ShowMessage(s+CRLF+CRLF+DbEditWarn);
  {$ifdef DEBUG_HELPER} LDH('..Aktualizace seznamu z '+frmMain_sDbFile.Caption+', itemindex='+IntToStr(j)); {$endif}
  frmMain_lTracks.Items.Text:=FillTracksFromDb(frmMain_sDbFile.Caption); //upravime zaznam v seznamu stop (jmeno souboru je v sTrackFile)
  frmMain_lTracks.ItemIndex:=j;
  frmMain_lTracks.Selected[j]:=true;
  lTracksOnClick(nil);
end;

procedure lPointsOnClick(Sender: TObject); //vypocet vzdalenosti mezi oznacenymi body
var s,sText:string;
    lat,lon,lat_last,lon_last,dist:extended;
    i:integer;
begin
  lat_last:=500;
  dist:=0;
  for i:=0 to frmTrack_lPoints.Items.Count-1 do begin
    if(not frmTrack_lPoints.Selected[i]) then continue;
    sText:=frmTrack_lPoints.Items[i];
    Fetch(sText,',');
    s:=Fetch(sText,',');
    //{$ifdef DEBUG_HELPER} LDH('Lat:= '+s); {$endif}
    lat:=StrToFloat(s);
    s:=Fetch(sText,',');
    //{$ifdef DEBUG_HELPER} LDH('Lon:= '+s); {$endif}
    lon:=StrToFloat(s);
    if(lat_last<>500) then begin
      dist:=dist+PointDistance(lat_last, lon_last, lat,lon);
      //{$ifdef DEBUG_HELPER} LDH('dist:= '+FloatToStr(dist)); {$endif}
    end;
    lat_last:=lat;
    lon_last:=lon;
  end;
  if(dist=0) then frmTrack_sSelectedDist.Caption:='0 m'
  else frmTrack_sSelectedDist.Caption:=ReplaceString(FormatFloat('#.# m',dist*1000),',','.');;
end;

procedure butPointSelectOnClick(Sender: TObject); //oznaci body podle nastaveni
var s,sTime,sAccuracy:string;
    i:integer;
    b:boolean;
    dt:TDateTime;
    accuracy,accLimit:extended;
begin
  if(frmTrack_rPointSelectType.ItemIndex<0) then exit;
  if(frmTrack_ePointAccuracy.Text='') then frmTrack_ePointAccuracy.Text:='0';
  accLimit:=StrToFloat(ReplaceString(frmTrack_ePointAccuracy.Text,',','.'));
  for i:=0 to frmTrack_lPoints.Items.Count-1 do begin
    b:=false;
    if(frmTrack_rPointSelectType.ItemIndex=0) then b:=true
    else begin
      //nacteme pole s casem a presnosti
      s:=frmTrack_lPoints.Items[i];
      {sKey:=}Fetch(s,',');
      {sLat:=}Fetch(s,',');
      {sLon:=}Fetch(s,',');
      sTime:=Fetch(s,',');
      {sAlt:=}Fetch(s,',');
      sAccuracy:=Fetch(s,',');
      case frmTrack_rPointSelectType.ItemIndex of
      1:  begin
            if(sAccuracy='NULL') or (sAccuracy='') then accuracy:=9999
            else accuracy:=StrToFloat(sAccuracy);
            if(accuracy>accLimit) then b:=true;
          end;
      2:  begin
            dt:=TextToDT(sTime);
            //{$ifdef DEBUG_HELPER} LDH('sTime='+sTime
            //  +', dt='+FloatToStr(dt)
            //  +', From='+FloatToStr(Trunc(frmTrack_dtDateFrom.Date)+Frac(frmTrack_dtTimeFrom.Time))); {$endif}
            if(dt>=(Trunc(frmTrack_dtDateFrom.Date)+Frac(frmTrack_dtTimeFrom.Time)))
              AND (dt<=(Trunc(frmTrack_dtDateTo.Date)+Frac(frmTrack_dtTimeTo.Time))) then b:=true;
          end;
      end;
    end;
    if(b) then frmTrack_lPoints.Selected[i]:=true;
  end;
  lPointsOnClick(nil);
end;

procedure SelectTracks(sIds:string); //oznaci pozadovane stopy
var sId:string;
    i,j:integer;
begin
  j:=-1;
  while sIds<>'' do begin
    sId:=Fetch(sIds,',');
    sId:=sId+',';
    for i:=0 to frmMain_lTracks.Items.Count-1 do begin
      if(Pos(sId,frmMain_lTracks.Items[i])=1) then begin
        if(j<0) then j:=i;
        frmMain_lTracks.Selected[i]:=true;
      end;
    end;
  end;
  frmMain_lTracks.ItemIndex:=j;
  lTracksOnClick(nil); //klikneme na polozku v seznamu, zobrazime vlastnosti stopy
end;

function SplitTrack(sId,sKey:string):string;
 //rozdelit stopu sId, bod s Indexem sIdx bude prvni v nove vytvorene stope
 //vraci ID nove vytvorene casti stopy
var sIdNew,sSql,sName:string;
    DB:TSqliteDatabase;
begin
  DB:=TSqliteDatabase.Create(GetADrakeDbPath());
  try
    DB.Start('GgDrake_Split',''); //zahajime transakci
    //zjistime jmeno, datumm a cas pro novou stopu
    sSql:='SELECT date||"_"||substr("000000"||time,-6,6) FROM track_log WHERE trackid='+sId+' AND key='+sKey;
    sName:=DB.GetTableString(sSql);
    //{$ifdef DEBUG_HELPER} LDH('jmeno nove casti stopy (datum,cas prvniho bodu): '+sName); {$endif}
    //vytvorime zaznam v tabulce s hlavickou a zjistime id nove stopy
    sSql:='INSERT INTO track_h (name,date,time,size,odo) '
      +'VALUES("track_'+sName+'",'+ReplaceString(sName,'_',',')+',0,0)';
    DB.ExecSQL(sSql);
    //{$ifdef DEBUG_HELPER} LDH('vytvoreni hlavicky nove stopy: '+sSql); {$endif}
    sSql:='SELECT trackid FROM track_h WHERE name="track_'+sName+'"';
    sIdNew:=DB.GetTableString(sSql);
    //{$ifdef DEBUG_HELPER} LDH('ID nove stopy: '+sIdNew+', ('+sSql+')'); {$endif}
    //vsem nasledujicim bodum zmenime ID stopy na nove
    sSql:='UPDATE track_log SET trackid='+sIdNew+' WHERE trackid='+sId+' AND key>='+sKey;
    DB.ExecSQL(sSql);
    //{$ifdef DEBUG_HELPER} LDH('Zmena ID v bodech stopy: '+sSql); {$endif}
    //upravime pocet bodu stopy a jeji stopy v hlavicce
    UpdateTracksHeader(DB,sId+','+sIdNew);
    //ukoncime transakci
    DB.Commit('GgDrake_Split');//ukoncime transakci
  finally
    if(DB<>nil) then DB.Free();
  end;
  Result:=sIdNew;
end;

procedure butPointSplitOnClick(Sender: TObject);
var s,sKey,sIdNew:string;
    i,j:integer;
begin
  i:=frmTrack_lPoints.ItemIndex;
  if(i<0) then begin
    ShowMessage(_('Neni vybrany bod, v kterem by mela byt stopa rozdelena.')); //##_EN_##
    exit;
  end;
  s:=frmTrack_lPoints.Items[i];
  sKey:=Fetch(s,',');
  j:=MessBox(_('Rozdelit stopu pred bodem s Key=')+sKey+'?',   //##_EN_##
    _('Rozdeleni stopy v oznacenem bode'),4);                  //##_EN_##
  if(j<>6) then exit;
  frmMain_memoLog.Lines.Add('PointSplit: start');
  sIdNew:=SplitTrack(GetTrackId(),sKey); //rozdelit stopu sId pred bodem s indexem sIdx
  frmMain_memoLog.Lines.Add(LOG_TAB+_('The track')+' ID='+GetTrackId()+' '+_('was split, the new part have')+' ID='+sIdNew); //##_EN_##
  //aktualizujeme seznam stop a oznacime rozdelene stopy
  frmMain_lTracks.Items.Text:=FillTracksFromDb(GetADrakeDbPath());
  s:=GetTrackId()+', '+sIdNew;
  SelectTracks(s);
  frmMain_memoLog.Lines.Add('PointSplit: '+_('old+new tracks')+' ID: '+s);  //##_EN_##
end;

function GetKeyForSplit(sId:string):string; //najde bod, pred kterym ma byt stopa rozdelena
var sSql,sDt,sTm,sKey:string;
    dtStart,dtStop,dtDel,dtLimit:TDateTime;
    eLat,eLon,eDist,eDistLimit:extended;
    DB:TSqliteDatabase;
    tab:TSQLiteTable;
begin
  Result:=''; //nedelit v zadnem bode
  sKey:='';
  dtLimit:=StrToFloat(frmTrack_eSplitHod.Text)/24.+StrToFloat(frmTrack_eSplitMin.Text)/(24.*60.);
  eDistLimit:=StrToFloat(frmTrack_eSplitDist.Text);
  DB:=TSqliteDatabase.Create(GetADrakeDbPath());
  try
    sSql:='SELECT date, time, lat, lon, key FROM track_log WHERE trackid="'+sId+'" ORDER BY trackid,key;';
    tab:=DB.GetTable(sSql,false);
    try
      eDist:=0.;
      sDt:=tab.FieldAsString(0);
      sTm:=tab.FieldAsString(1);
      dtStart:=MakeTrackDatetime(sDt,sTm);
      eLat:=tab.FieldAsDouble(2);
      eLon:=tab.FieldAsDouble(3);
      tab.Next();
      while ((not tab.eof) and (Result='')) do begin
        //cas bodu a rozdil od startu
        sDt:=tab.FieldAsString(0);
        sTm:=tab.FieldAsString(1);
        dtStop:=MakeTrackDatetime(sDt,sTm);
        dtDel:=dtStop-dtStart;
        //vzdalenost
        eDist:=eDist+(PointDistance(eLat, eLon, tab.FieldAsDouble(2), tab.FieldAsDouble(3))*1000.);
        eLat:=tab.FieldAsDouble(2);
        eLon:=tab.FieldAsDouble(3);
        sKey:=tab.FieldAsString(4);
        //zjistime, zda je prekrocen limit
        //{$ifdef DEBUG_HELPER} LDH('Key='+sKey+', Dist='+FloatToStr(eDist)+', dtDel='+FloatToStr(24.*dtDel)); {$endif}
        if((frmTrack_rSplit.ItemIndex=2) and (dtDel>dtLimit))
          or ((frmTrack_rSplit.ItemIndex=1) and (eDist>eDistLimit)) then begin
          //{$ifdef DEBUG_HELPER} LDH('XXXXXXXXXXXX deleni stopy'); {$endif}//  break;
          Result:=sKey;
        end;
        tab.Next();
      end;
    finally
      if(tab<>nil) then tab.Free();
    end;
  finally
    if(DB<>nil) then DB.Free();
  end;
end;

procedure butSplitOnClick(Sender: TObject); //rozdeleni stopy po case nebo vzdalenosti
var sKey,sId,sIds,sIdNew:string;
begin
  if(frmTrack_rSplit.ItemIndex=0) then begin
    InitEditTrack(1);
    exit;
  end;
  sId:=GetTrackId();
  sIds:=sId;
  sKey:='1';

  frmMain_memoLog.Lines.Add('Split: start');
  while(sKey<>'') do begin
    //najdeme bod (sKey), pred kterym stopu rozdelit a zavolame deleni SplitTrack(sId,sKey)
    sKey:=GetKeyForSplit(sId);
    if(sKey<>'') then begin
      sIdNew:=SplitTrack(sId,sKey); //rozdelit stopu sId pred bodem s indexem sIdx
      frmMain_memoLog.Lines.Add(LOG_TAB+_('the new part have ID=')+sIdNew); //##_EN_##
      sId:=sIdNew;
      sIds:=sIds+','+sId;
    end;
  end;
  frmMain_memoLog.Lines.Add('Split: ok');
  frmMain_lTracks.Items.Text:=FillTracksFromDb(GetADrakeDbPath());
  SelectTracks(sIds);
end;

procedure butViewPointsOnClick(Sender: TObject);
begin
  InitEditTrack(1);
end;

procedure butTrackRenameOnClick(Sender: TObject);
var s,sName,sId:string;
    i:integer;
    DB:TSqliteDatabase;
begin
  i:=frmMain_lTracks.ItemIndex;
  sName:=frmMain_sTrackName.Caption;
  sId:=Fetch(sName,':');
  s:=Trim(InputBox(_('Prejmenovani stopy'),_('Nove jmeno:'),sName));
  if(s=sName) then exit;
  sName:='"'+sName+'"';
  frmMain_lTracks.Items[i]:=ReplaceString(frmMain_lTracks.Items[i],sName,'"'+s+'"');
  frmMain_lTracks.ItemIndex:=i;
  frmMain_lTracks.Selected[i]:=true;
  frmMain_sTrackName.Caption:=sId+': '+s;
  //jeste promitneme nove jmeno do databaze
  DB:=TSqliteDatabase.Create(GetADrakeDbPath());
  try
    DB.ExecSQL('UPDATE track_h SET name='''+s+''' WHERE trackid='+sId);
    frmMain_memoLog.Lines.Add(_('The track')+' ID='+sId+' was renamed');  //##_EN_##
  finally
    if(DB<>nil) then DB.Free();
  end;
end;

procedure butTrackDelOnClick(Sender: TObject);
var s,sId:string;
    i,j:integer;
    DB:TSqliteDatabase;
begin
  DB:=TSqliteDatabase.Create(GetADrakeDbPath());
  try
    DB.Start('GgDrake_Del',''); //zahajime transakci
    j:=0;
    for i:=frmMain_lTracks.Items.Count-1 downto 0 do begin
      if(frmMain_lTracks.Selected[i]=false) then continue;
      s:=frmMain_lTracks.Items[i];
      sId:=Fetch(s,',');
      frmMain_lTracks.Items.Delete(i);
      DB.ExecSQL('DELETE FROM track_log WHERE trackid='+sId);
      DB.ExecSQL('DELETE FROM track_h WHERE trackid='+sId);
      frmMain_memoLog.Lines.Add(_('The track')+' ID='+sId+' was deleted');    //##_EN_##
      Inc(j);
    end;
    DB.Commit('GgDrake_Del');//ukoncime transakci
  finally
    if(DB<>nil) then DB.Free();
  end;
  ShowMessage(_('Pocet smazanych stop:')+' '+IntToStr(j));
end;

procedure butTrackToADOnClick(Sender: TObject);
var s,sDev,sSrc,sTrg:string;
begin
  //pokud pracujeme s databazi nactenou z disku bez predchoziho nacteni z telefonu,
  //zobrazime varovani
  if(frmMain_bTrackViewFromDb.Enabled=false) and (frmMain_bTrackViewFromDb.Checked=true) then begin
    if(MessBox(_('Databaze obsahuje krome stop i koncepty logu, nastaveni filtru a informace o TB/GC.')+CRLF+CRLF
      +_('Prekopirovanim databaze drive ulozene do PC muze dojit k nechtene zmene techto dat v telefonu.'),
      _('Varovani pri prepisu databaze'),1)<>1) then exit;    //##_EN_##
  end;
  frmMain_memoLog.Lines.Add('Copy _adrake.db3: start');
  s:=frmMain_sMtpPath.Caption;
  sDev:=Fetch(s,':');
  sTrg:='/'+s{+'/_adrake.db3'};  //nesmi byt zadany cislovy soubor, jen adresar!!!, jinak to pada
  sTrg:=ReplaceString(sTrg,'/','\');
  {$ifdef DEBUG_HELPER} LDH('SetDevice: '+sDev); {$endif}
  ML_SetDeviceName(sDev);
  sSrc:=GetADrakeDbPath();
  {$ifdef DEBUG_HELPER} LDH('src: '+sSrc+', trg: '+sTrg); {$endif}
  ML_PutFile(sSrc,sTrg,false); //vzdy prepiseme
  frmMain_memoLog.Lines.Add('Copy _adrake.db3: ok');
end;

procedure butTracksJoinOnClick(Sender: TObject);
var s,sId,sId1:string;
    i,j:integer;
    DB:TSqliteDatabase;
begin
  frmMain_memoLog.Lines.Add('TracksJoin: start');
  j:=0;
  DB:=TSqliteDatabase.Create(GetADrakeDbPath());
  try
    DB.Start('GgDrake_Join',''); //zahajime transakci
    //postupujeme od konce, to je nejmensi ID a tedy i nejmensi datum/cas
    for i:=(frmMain_lTracks.Items.Count-1) downto 0 do begin
      if(frmMain_lTracks.Selected[i]) then begin
        s:=frmMain_lTracks.Items[i];
        sId:=Fetch(s,',');
        if(sId1='') then sId1:=sId //prvni stopa
        else begin  //upravime bodum stopy ID a hlavicku mazane stopy smazeme
          DB.ExecSQL('UPDATE track_log SET trackid='+sId1+' WHERE trackid='+sId);
          DB.ExecSQL('DELETE FROM track_h WHERE trackid='+sId);
          frmMain_lTracks.Items.Delete(i);  //vymazeme stopu ze seznamu
          frmMain_memoLog.Lines.Add(LOG_TAB+_('joined track')+' ID='+sId);    //##_EN_##
        end;
        Inc(j);
      end;
    end;
    UpdateTracksHeader(DB,sId1); //aktualizovat zaznam stopy v hlavicce
    DB.Commit('GgDrake_Join'); //ukoncime transakci
  finally
    DB.Free;
  end;
  frmMain_lTracks.Items.Text:=FillTracksFromDb(GetADrakeDbPath());
  //ShowMessage(_('Pocet spojenych stop:')+' '+IntToStr(j));
  frmMain_memoLog.Lines.Add('TracksJoin: ok'+' ('+_('pocet pripojenych stop:')+' '+IntToStr(j)+')');    //##_EN_##
  SelectTracks(sId1);
end;

procedure lPointsOnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var i,j:Integer;
    l:TListbox;
begin
  if((Shift=[ssCtrl]) and (Key=65)) then begin   //obsluha CRTL+A
    l:=(Sender as TListBox);
    j:=l.ItemIndex;
    for i:=0 to (l.Items.Count-1) do l.Selected[i]:=true;
    l.ItemIndex:=j;
  end;
  lPointsOnClick(nil);
end;

